@@ -1,10 +1,17 @@
 from bleak.backends.descriptor import BleakGATTDescriptor
+from bleak.uuids import uuidstr_to_str
 
 
 class BleakGATTDescriptorBlueZDBus(BleakGATTDescriptor):
     """GATT Descriptor implementation for BlueZ DBus backend"""
 
-    def __init__(self, obj: dict, object_path: str, characteristic_uuid: str, characteristic_handle: int):
+    def __init__(
+        self,
+        obj: dict,
+        object_path: str,
+        characteristic_uuid: str,
+        characteristic_handle: int,
+    ):
         super(BleakGATTDescriptorBlueZDBus, self).__init__(obj)
         self.__path = object_path
         self.__characteristic_uuid = characteristic_uuid
@@ -35,3 +42,8 @@ class BleakGATTDescriptorBlueZDBus(BleakGATTDescriptor):
     def path(self) -> str:
         """The DBus path. Mostly needed by `bleak`, not by end user"""
         return self.__path
+
+    @property
+    def description(self) -> str:
+        """Description for this descriptor"""
+        return uuidstr_to_str(self.uuid)
