@@ -27,9 +27,7 @@ class GattCharacteristicsFlags(enum.Enum):
 
 
 class BleakGATTCharacteristic(abc.ABC):
-    """Interface for the Bleak representation of a GATT Characteristic
-
-    """
+    """Interface for the Bleak representation of a GATT Characteristic"""
 
     def __init__(self, obj: Any):
         self.obj = obj
