@@ -41,8 +41,12 @@ logger = logging.getLogger(__name__)
 
 CBCentralManagerDelegate = objc.protocolNamed("CBCentralManagerDelegate")
 
-_mac_version = list(map(int, platform.mac_ver()[0].split(".")))
-_IS_PRE_10_13 = _mac_version[0] == 10 and _mac_version[1] < 13
+try:
+    _mac_version = list(map(int, platform.mac_ver()[0].split(".")))
+    _IS_PRE_10_13 = _mac_version[0] == 10 and _mac_version[1] < 13
+except:  # noqa For building docs
+    _mac_version = ""
+    _IS_PRE_10_13 = False
 
 
 class CMDConnectionState(Enum):
@@ -74,9 +78,6 @@ class CentralManagerDelegate(NSObject):
         self.callbacks = {}
         self.disconnected_callback = None
 
-        if not self.compliant():
-            logger.warning("CentralManagerDelegate is not compliant")
-
         self.central_manager = CBCentralManager.alloc().initWithDelegate_queue_(
             self, dispatch_queue_create(b"bleak.corebluetooth", DISPATCH_QUEUE_SERIAL)
         )
@@ -85,12 +86,6 @@ class CentralManagerDelegate(NSObject):
 
     # User defined functions
 
-    def compliant(self):
-        """Determines whether the class adheres to the CBCentralManagerDelegate protocol"""
-        return CentralManagerDelegate.pyobjc_classMethods.conformsToProtocol_(
-            CBCentralManagerDelegate
-        )
-
     @property
     def isConnected(self) -> bool:
         return self._connection_state == CMDConnectionState.CONNECTED
@@ -105,11 +100,8 @@ class CentralManagerDelegate(NSObject):
         """
         await asyncio.wait_for(self.powered_on_event.wait(), timeout)
 
-    async def scanForPeripherals_(self, scan_options) -> List[CBPeripheral]:
-        """
-        Scan for peripheral devices
-        scan_options = { service_uuids, timeout }
-        """
+    @objc.python_method
+    def start_scan(self, scan_options):
         # remove old
         self.devices = {}
         service_uuids = []
@@ -119,17 +111,12 @@ class CentralManagerDelegate(NSObject):
                 list(map(string2uuid, service_uuids_str))
             )
 
-        timeout = 0
-        if "timeout" in scan_options:
-            timeout = float(scan_options["timeout"])
-
         self.central_manager.scanForPeripheralsWithServices_options_(
             service_uuids, None
         )
 
-        if timeout > 0:
-            await asyncio.sleep(timeout)
-
+    @objc.python_method
+    async def stop_scan(self) -> List[CBPeripheral]:
         self.central_manager.stopScan()
 
         # Wait a while to allow central manager to stop scanning.
@@ -144,6 +131,17 @@ class CentralManagerDelegate(NSObject):
 
         return []
 
+    @objc.python_method
+    async def scanForPeripherals_(self, scan_options) -> List[CBPeripheral]:
+        """
+        Scan for peripheral devices
+        scan_options = { service_uuids, timeout }
+        """
+
+        self.start_scan(scan_options)
+        await asyncio.sleep(float(scan_options.get("timeout", 0.0)))
+        return await self.stop_scan()
+
     async def connect_(self, peripheral: CBPeripheral) -> bool:
         self._connection_state = CMDConnectionState.PENDING
         self.central_manager.connectPeripheral_options_(peripheral, None)
@@ -201,6 +199,8 @@ class CentralManagerDelegate(NSObject):
         RSSI: NSNumber,
     ):
         # Note: this function might be called several times for same device.
+        # This can happen for instance when an active scan is done, and the
+        # second call with contain the data from the BLE scan response.
         # Example a first time with the following keys in advertisementData:
         # ['kCBAdvDataLocalName', 'kCBAdvDataIsConnectable', 'kCBAdvDataChannel']
         # ... and later a second time with other keys (and values) such as:
@@ -216,6 +216,9 @@ class CentralManagerDelegate(NSObject):
 
         if uuid_string in self.devices:
             device = self.devices[uuid_string]
+            # It could be the device did not have a name previously but now it does.
+            if peripheral.name():
+                device.name = peripheral.name()
         else:
             address = uuid_string
             name = peripheral.name() or None
@@ -255,9 +258,12 @@ class CentralManagerDelegate(NSObject):
                 peripheral.identifier().UUIDString()
             )
         )
-        peripheralDelegate = PeripheralDelegate.alloc().initWithPeripheral_(peripheral)
-        self.connected_peripheral_delegate = peripheralDelegate
-        self._connection_state = CMDConnectionState.CONNECTED
+        if self._connection_state != CMDConnectionState.CONNECTED:
+            peripheralDelegate = PeripheralDelegate.alloc().initWithPeripheral_(
+                peripheral
+            )
+            self.connected_peripheral_delegate = peripheralDelegate
+            self._connection_state = CMDConnectionState.CONNECTED
 
     def centralManager_didConnectPeripheral_(self, central, peripheral):
         logger.debug("centralManager_didConnectPeripheral_")
@@ -289,6 +295,8 @@ class CentralManagerDelegate(NSObject):
         self, central: CBCentralManager, peripheral: CBPeripheral, error: NSError
     ):
         logger.debug("Peripheral Device disconnected!")
+        self.connected_peripheral_delegate = None
+        self.connected_peripheral = None
         self._connection_state = CMDConnectionState.DISCONNECTED
 
         if self.disconnected_callback is not None:
